@@ -38,10 +38,12 @@ Partial matches are possible::
     m.variables().get('operator2')  # Returns "add"
 
 """
+from __future__ import annotations
+
 import re
 from typing import Callable, Dict, Iterable, Iterator, List
 from typing import Match as RegexMatch
-from typing import Optional, Pattern, Tuple, cast
+from typing import Optional, Pattern, Tuple
 
 from .regex_parser import (
     AnyNode,
@@ -81,16 +83,15 @@ class _CompiledGrammar:
     def __init__(
         self,
         root_node: Node,
-        escape_funcs: Optional[EscapeFuncDict] = None,
-        unescape_funcs: Optional[EscapeFuncDict] = None,
+        escape_funcs: EscapeFuncDict | None = None,
+        unescape_funcs: EscapeFuncDict | None = None,
     ) -> None:
-
         self.root_node = root_node
         self.escape_funcs = escape_funcs or {}
         self.unescape_funcs = unescape_funcs or {}
 
         #: Dictionary that will map the regex names to Node instances.
-        self._group_names_to_nodes: Dict[
+        self._group_names_to_nodes: dict[
             str, str
         ] = {}  # Maps regex group names to varnames.
         counter = [0]
@@ -118,7 +119,8 @@ class _CompiledGrammar:
         # input contains some additional characters at the end that don't match the grammar.)
         self._re_prefix_with_trailing_input = [
             re.compile(
-                r"(?:%s)(?P<%s>.*?)$" % (t.rstrip("$"), _INVALID_TRAILING_INPUT), flags
+                r"(?:{})(?P<{}>.*?)$".format(t.rstrip("$"), _INVALID_TRAILING_INPUT),
+                flags,
             )
             for t in self._re_prefix_patterns
         ]
@@ -168,21 +170,31 @@ class _CompiledGrammar:
 
             # A `Variable` wraps the children into a named group.
             elif isinstance(node, Variable):
-                return "(?P<%s>%s)" % (
+                return "(?P<{}>{})".format(
                     create_group_func(node),
                     transform(node.childnode),
                 )
 
             # `Repeat`.
             elif isinstance(node, Repeat):
-                return "(?:%s){%i,%s}%s" % (
+                if node.max_repeat is None:
+                    if node.min_repeat == 0:
+                        repeat_sign = "*"
+                    elif node.min_repeat == 1:
+                        repeat_sign = "+"
+                else:
+                    repeat_sign = "{%i,%s}" % (
+                        node.min_repeat,
+                        ("" if node.max_repeat is None else str(node.max_repeat)),
+                    )
+
+                return "(?:{}){}{}".format(
                     transform(node.childnode),
-                    node.min_repeat,
-                    ("" if node.max_repeat is None else str(node.max_repeat)),
+                    repeat_sign,
                     ("" if node.greedy else "?"),
                 )
             else:
-                raise TypeError("Got %r" % (node,))
+                raise TypeError(f"Got {node!r}")
 
         return transform(root_node)
 
@@ -194,39 +206,113 @@ class _CompiledGrammar:
         Yield all the regular expressions matching a prefix of the grammar
         defined by the `Node` instance.
 
-        This can yield multiple expressions, because in the case of on OR
-        operation in the grammar, we can have another outcome depending on
-        which clause would appear first. E.g. "(A|B)C" is not the same as
-        "(B|A)C" because the regex engine is lazy and takes the first match.
-        However, because we the current input is actually a prefix of the
-        grammar which might not yet contain the data for "C", we need to know
-        both intermediate states, in order to call the appropriate
-        autocompletion for both cases.
+        For each `Variable`, one regex pattern will be generated, with this
+        named group at the end. This is required because a regex engine will
+        terminate once a match is found. For autocompletion however, we need
+        the matches for all possible paths, so that we can provide completions
+        for each `Variable`.
+
+        - So, in the case of an `Any` (`A|B|C)', we generate a pattern for each
+          clause. This is one for `A`, one for `B` and one for `C`. Unless some
+          groups don't contain a `Variable`, then these can be merged together.
+        - In the case of a `NodeSequence` (`ABC`), we generate a pattern for
+          each prefix that ends with a variable, and one pattern for the whole
+          sequence. So, that's one for `A`, one for `AB` and one for `ABC`.
 
         :param root_node: The :class:`Node` instance for which we generate the grammar.
         :param create_group_func: A callable which takes a `Node` and returns the next
             free name for this node.
         """
 
+        def contains_variable(node: Node) -> bool:
+            if isinstance(node, Regex):
+                return False
+            elif isinstance(node, Variable):
+                return True
+            elif isinstance(node, (Lookahead, Repeat)):
+                return contains_variable(node.childnode)
+            elif isinstance(node, (NodeSequence, AnyNode)):
+                return any(contains_variable(child) for child in node.children)
+
+            return False
+
         def transform(node: Node) -> Iterable[str]:
-            # Generate regexes for all permutations of this OR. Each node
-            # should be in front once.
+            # Generate separate pattern for all terms that contain variables
+            # within this OR. Terms that don't contain a variable can be merged
+            # together in one pattern.
             if isinstance(node, AnyNode):
+                # If we have a definition like:
+                #           (?P<name> .*)  | (?P<city> .*)
+                # Then we want to be able to generate completions for both the
+                # name as well as the city. We do this by yielding two
+                # different regular expressions, because the engine won't
+                # follow multiple paths, if multiple are possible.
+                children_with_variable = []
+                children_without_variable = []
                 for c in node.children:
-                    for r in transform(c):
-                        yield "(?:%s)?" % r
+                    if contains_variable(c):
+                        children_with_variable.append(c)
+                    else:
+                        children_without_variable.append(c)
 
-            # For a sequence. We can either have a match for the sequence
-            # of all the children, or for an exact match of the first X
-            # children, followed by a partial match of the next children.
+                for c in children_with_variable:
+                    yield from transform(c)
+
+                # Merge options without variable together.
+                if children_without_variable:
+                    yield "|".join(
+                        r for c in children_without_variable for r in transform(c)
+                    )
+
+            # For a sequence, generate a pattern for each prefix that ends with
+            # a variable + one pattern of the complete sequence.
+            # (This is because, for autocompletion, we match the text before
+            # the cursor, and completions are given for the variable that we
+            # match right before the cursor.)
             elif isinstance(node, NodeSequence):
-                for i in range(len(node.children)):
-                    a = [
-                        cls._transform(c, create_group_func) for c in node.children[:i]
-                    ]
+                # For all components in the sequence, compute prefix patterns,
+                # as well as full patterns.
+                complete = [cls._transform(c, create_group_func) for c in node.children]
+                prefixes = [list(transform(c)) for c in node.children]
+                variable_nodes = [contains_variable(c) for c in node.children]
 
-                    for c_str in transform(node.children[i]):
-                        yield "(?:%s)" % ("".join(a) + c_str)
+                # If any child is contains a variable, we should yield a
+                # pattern up to that point, so that we are sure this will be
+                # matched.
+                for i in range(len(node.children)):
+                    if variable_nodes[i]:
+                        for c_str in prefixes[i]:
+                            yield "".join(complete[:i]) + c_str
+
+                # If there are non-variable nodes, merge all the prefixes into
+                # one pattern. If the input is: "[part1] [part2] [part3]", then
+                # this gets compiled into:
+                #  (complete1 + (complete2 + (complete3  | partial3) | partial2) | partial1 )
+                # For nodes that contain a variable, we skip the "|partial"
+                # part here, because thees are matched with the previous
+                # patterns.
+                if not all(variable_nodes):
+                    result = []
+
+                    # Start with complete patterns.
+                    for i in range(len(node.children)):
+                        result.append("(?:")
+                        result.append(complete[i])
+
+                    # Add prefix patterns.
+                    for i in range(len(node.children) - 1, -1, -1):
+                        if variable_nodes[i]:
+                            # No need to yield a prefix for this one, we did
+                            # the variable prefixes earlier.
+                            result.append(")")
+                        else:
+                            result.append("|(?:")
+                            # If this yields multiple, we should yield all combinations.
+                            assert len(prefixes[i]) == 1
+                            result.append(prefixes[i][0])
+                            result.append("))")
+
+                    yield "".join(result)
 
             elif isinstance(node, Regex):
                 yield "(?:%s)?" % node.regex
@@ -243,7 +329,7 @@ class _CompiledGrammar:
                 # (Note that we should not append a '?' here. the 'transform'
                 # method will already recursively do that.)
                 for c_str in transform(node.childnode):
-                    yield "(?P<%s>%s)" % (create_group_func(node), c_str)
+                    yield f"(?P<{create_group_func(node)}>{c_str})"
 
             elif isinstance(node, Repeat):
                 # If we have a repetition of 8 times. That would mean that the
@@ -251,25 +337,28 @@ class _CompiledGrammar:
                 # match, followed by a partial match.
                 prefix = cls._transform(node.childnode, create_group_func)
 
-                for c_str in transform(node.childnode):
-                    if node.max_repeat:
-                        repeat_sign = "{,%i}" % (node.max_repeat - 1)
-                    else:
-                        repeat_sign = "*"
-                    yield "(?:%s)%s%s(?:%s)?" % (
-                        prefix,
-                        repeat_sign,
-                        ("" if node.greedy else "?"),
-                        c_str,
-                    )
+                if node.max_repeat == 1:
+                    yield from transform(node.childnode)
+                else:
+                    for c_str in transform(node.childnode):
+                        if node.max_repeat:
+                            repeat_sign = "{,%i}" % (node.max_repeat - 1)
+                        else:
+                            repeat_sign = "*"
+                        yield "(?:{}){}{}{}".format(
+                            prefix,
+                            repeat_sign,
+                            ("" if node.greedy else "?"),
+                            c_str,
+                        )
 
             else:
                 raise TypeError("Got %r" % node)
 
         for r in transform(root_node):
-            yield "^%s$" % r
+            yield "^(?:%s)$" % r
 
-    def match(self, string: str) -> Optional["Match"]:
+    def match(self, string: str) -> Match | None:
         """
         Match the string with the grammar.
         Returns a :class:`Match` instance or `None` when the input doesn't match the grammar.
@@ -284,7 +373,7 @@ class _CompiledGrammar:
             )
         return None
 
-    def match_prefix(self, string: str) -> Optional["Match"]:
+    def match_prefix(self, string: str) -> Match | None:
         """
         Do a partial match of the string with the grammar. The returned
         :class:`Match` instance can contain multiple representations of the
@@ -303,6 +392,7 @@ class _CompiledGrammar:
                 return Match(
                     string, matches2, self._group_names_to_nodes, self.unescape_funcs
                 )
+
         return None
 
 
@@ -316,40 +406,40 @@ class Match:
     def __init__(
         self,
         string: str,
-        re_matches: List[Tuple[Pattern[str], RegexMatch[str]]],
-        group_names_to_nodes: Dict[str, str],
-        unescape_funcs: Dict[str, Callable[[str], str]],
+        re_matches: list[tuple[Pattern[str], RegexMatch[str]]],
+        group_names_to_nodes: dict[str, str],
+        unescape_funcs: dict[str, Callable[[str], str]],
     ):
         self.string = string
         self._re_matches = re_matches
         self._group_names_to_nodes = group_names_to_nodes
         self._unescape_funcs = unescape_funcs
 
-    def _nodes_to_regs(self) -> List[Tuple[str, Tuple[int, int]]]:
+    def _nodes_to_regs(self) -> list[tuple[str, tuple[int, int]]]:
         """
         Return a list of (varname, reg) tuples.
         """
 
-        def get_tuples() -> Iterable[Tuple[str, Tuple[int, int]]]:
+        def get_tuples() -> Iterable[tuple[str, tuple[int, int]]]:
             for r, re_match in self._re_matches:
                 for group_name, group_index in r.groupindex.items():
                     if group_name != _INVALID_TRAILING_INPUT:
-                        regs = cast(Tuple[Tuple[int, int], ...], re_match.regs)
+                        regs = re_match.regs
                         reg = regs[group_index]
                         node = self._group_names_to_nodes[group_name]
                         yield (node, reg)
 
         return list(get_tuples())
 
-    def _nodes_to_values(self) -> List[Tuple[str, str, Tuple[int, int]]]:
+    def _nodes_to_values(self) -> list[tuple[str, str, tuple[int, int]]]:
         """
-        Returns list of list of (Node, string_value) tuples.
+        Returns list of (Node, string_value) tuples.
         """
 
-        def is_none(sl: Tuple[int, int]) -> bool:
+        def is_none(sl: tuple[int, int]) -> bool:
             return sl[0] == -1 and sl[1] == -1
 
-        def get(sl: Tuple[int, int]) -> str:
+        def get(sl: tuple[int, int]) -> str:
             return self.string[sl[0] : sl[1]]
 
         return [
@@ -362,7 +452,7 @@ class Match:
         unwrapper = self._unescape_funcs.get(varname)
         return unwrapper(value) if unwrapper else value
 
-    def variables(self) -> "Variables":
+    def variables(self) -> Variables:
         """
         Returns :class:`Variables` instance.
         """
@@ -370,13 +460,13 @@ class Match:
             [(k, self._unescape(k, v), sl) for k, v, sl in self._nodes_to_values()]
         )
 
-    def trailing_input(self) -> Optional["MatchVariable"]:
+    def trailing_input(self) -> MatchVariable | None:
         """
         Get the `MatchVariable` instance, representing trailing input, if there is any.
         "Trailing input" is input at the end that does not match the grammar anymore, but
         when this is removed from the end of the input, the input would be a valid string.
         """
-        slices: List[Tuple[int, int]] = []
+        slices: list[tuple[int, int]] = []
 
         # Find all regex group for the name _INVALID_TRAILING_INPUT.
         for r, re_match in self._re_matches:
@@ -392,7 +482,7 @@ class Match:
             return MatchVariable("<trailing_input>", value, slice)
         return None
 
-    def end_nodes(self) -> Iterable["MatchVariable"]:
+    def end_nodes(self) -> Iterable[MatchVariable]:
         """
         Yields `MatchVariable` instances for all the nodes having their end
         position at the end of the input string.
@@ -405,27 +495,27 @@ class Match:
 
 
 class Variables:
-    def __init__(self, tuples: List[Tuple[str, str, Tuple[int, int]]]) -> None:
+    def __init__(self, tuples: list[tuple[str, str, tuple[int, int]]]) -> None:
         #: List of (varname, value, slice) tuples.
         self._tuples = tuples
 
     def __repr__(self) -> str:
-        return "%s(%s)" % (
+        return "{}({})".format(
             self.__class__.__name__,
-            ", ".join("%s=%r" % (k, v) for k, v, _ in self._tuples),
+            ", ".join(f"{k}={v!r}" for k, v, _ in self._tuples),
         )
 
-    def get(self, key: str, default: Optional[str] = None) -> Optional[str]:
+    def get(self, key: str, default: str | None = None) -> str | None:
         items = self.getall(key)
         return items[0] if items else default
 
-    def getall(self, key: str) -> List[str]:
+    def getall(self, key: str) -> list[str]:
         return [v for k, v, _ in self._tuples if k == key]
 
-    def __getitem__(self, key: str) -> Optional[str]:
+    def __getitem__(self, key: str) -> str | None:
         return self.get(key)
 
-    def __iter__(self) -> Iterator["MatchVariable"]:
+    def __iter__(self) -> Iterator[MatchVariable]:
         """
         Yield `MatchVariable` instances.
         """
@@ -443,7 +533,7 @@ class MatchVariable:
                   in the input string.
     """
 
-    def __init__(self, varname: str, value: str, slice: Tuple[int, int]) -> None:
+    def __init__(self, varname: str, value: str, slice: tuple[int, int]) -> None:
         self.varname = varname
         self.value = value
         self.slice = slice
@@ -452,13 +542,13 @@ class MatchVariable:
         self.stop = self.slice[1]
 
     def __repr__(self) -> str:
-        return "%s(%r, %r)" % (self.__class__.__name__, self.varname, self.value)
+        return f"{self.__class__.__name__}({self.varname!r}, {self.value!r})"
 
 
 def compile(
     expression: str,
-    escape_funcs: Optional[EscapeFuncDict] = None,
-    unescape_funcs: Optional[EscapeFuncDict] = None,
+    escape_funcs: EscapeFuncDict | None = None,
+    unescape_funcs: EscapeFuncDict | None = None,
 ) -> _CompiledGrammar:
     """
     Compile grammar (given as regex string), returning a `CompiledGrammar`
@@ -473,8 +563,8 @@ def compile(
 
 def _compile_from_parse_tree(
     root_node: Node,
-    escape_funcs: Optional[EscapeFuncDict] = None,
-    unescape_funcs: Optional[EscapeFuncDict] = None,
+    escape_funcs: EscapeFuncDict | None = None,
+    unescape_funcs: EscapeFuncDict | None = None,
 ) -> _CompiledGrammar:
     """
     Compile grammar (given as parse tree), returning a `CompiledGrammar`
