@@ -7,11 +7,13 @@ NOTE: There is no `DynamicHistory`:
       loading can be done asynchronously and making the history swappable would
       probably break this.
 """
-import asyncio
+from __future__ import annotations
+
 import datetime
 import os
 import threading
 from abc import ABCMeta, abstractmethod
+from asyncio import get_running_loop
 from typing import AsyncGenerator, Iterable, List, Optional, Sequence, Tuple
 
 __all__ = [
@@ -36,7 +38,7 @@ class History(metaclass=ABCMeta):
 
         # History that's loaded already, in reverse order. Latest, most recent
         # item first.
-        self._loaded_strings: List[str] = []
+        self._loaded_strings: list[str] = []
 
     #
     # Methods expected by `Buffer`.
@@ -60,7 +62,7 @@ class History(metaclass=ABCMeta):
         for item in self._loaded_strings:
             yield item
 
-    def get_strings(self) -> List[str]:
+    def get_strings(self) -> list[str]:
         """
         Get the strings from the history that are loaded so far.
         (In order. Oldest item first.)
@@ -110,7 +112,7 @@ class ThreadedHistory(History):
 
         self.history = history
 
-        self._load_thread: Optional[threading.Thread] = None
+        self._load_thread: threading.Thread | None = None
 
         # Lock for accessing/manipulating `_loaded_strings` and `_loaded`
         # together in a consistent state.
@@ -118,7 +120,7 @@ class ThreadedHistory(History):
 
         # Events created by each `load()` call. Used to wait for new history
         # entries from the loader thread.
-        self._string_load_events: List[threading.Event] = []
+        self._string_load_events: list[threading.Event] = []
 
     async def load(self) -> AsyncGenerator[str, None]:
         """
@@ -134,7 +136,7 @@ class ThreadedHistory(History):
             self._load_thread.start()
 
         # Consume the `_loaded_strings` list, using asyncio.
-        loop = asyncio.get_event_loop()
+        loop = get_running_loop()
 
         # Create threading Event so that we can wait for new items.
         event = threading.Event()
@@ -158,7 +160,7 @@ class ThreadedHistory(History):
                     continue
 
                 # Read new items (in lock).
-                def in_executor() -> Tuple[List[str], bool]:
+                def in_executor() -> tuple[list[str], bool]:
                     with self._lock:
                         new_items = self._loaded_strings[items_yielded:]
                         done = self._loaded
@@ -210,7 +212,7 @@ class ThreadedHistory(History):
         self.history.store_string(string)
 
     def __repr__(self) -> str:
-        return "ThreadedHistory(%r)" % (self.history,)
+        return f"ThreadedHistory({self.history!r})"
 
 
 class InMemoryHistory(History):
@@ -221,7 +223,7 @@ class InMemoryHistory(History):
     `append_string` for all items or pass a list of strings to `__init__` here.
     """
 
-    def __init__(self, history_strings: Optional[Sequence[str]] = None) -> None:
+    def __init__(self, history_strings: Sequence[str] | None = None) -> None:
         super().__init__()
         # Emulating disk storage.
         if history_strings is None:
@@ -230,8 +232,7 @@ class InMemoryHistory(History):
             self._storage = list(history_strings)
 
     def load_history_strings(self) -> Iterable[str]:
-        for item in self._storage[::-1]:
-            yield item
+        yield from self._storage[::-1]
 
     def store_string(self, string: str) -> None:
         self._storage.append(string)
@@ -260,11 +261,11 @@ class FileHistory(History):
 
     def __init__(self, filename: str) -> None:
         self.filename = filename
-        super(FileHistory, self).__init__()
+        super().__init__()
 
     def load_history_strings(self) -> Iterable[str]:
-        strings: List[str] = []
-        lines: List[str] = []
+        strings: list[str] = []
+        lines: list[str] = []
 
         def add() -> None:
             if lines:
