@@ -6,19 +6,16 @@ A lot of thanks, regarding outputting of colors, goes to the Pygments project:
 everything has been highly optimized.)
 http://pygments.org/
 """
-import array
-import errno
+from __future__ import annotations
+
 import io
 import os
 import sys
-from contextlib import contextmanager
 from typing import (
-    IO,
     Callable,
     Dict,
     Hashable,
     Iterable,
-    Iterator,
     List,
     Optional,
     Sequence,
@@ -27,12 +24,14 @@ from typing import (
     Tuple,
 )
 
+from prompt_toolkit.cursor_shapes import CursorShape
 from prompt_toolkit.data_structures import Size
 from prompt_toolkit.output import Output
 from prompt_toolkit.styles import ANSI_COLOR_NAMES, Attrs
 from prompt_toolkit.utils import is_dumb_terminal
 
 from .color_depth import ColorDepth
+from .flush_stdout import flush_stdout
 
 __all__ = [
     "Vt100_Output",
@@ -162,10 +161,10 @@ class _16ColorCache:
 
     def __init__(self, bg: bool = False) -> None:
         self.bg = bg
-        self._cache: Dict[Hashable, _ColorCodeAndName] = {}
+        self._cache: dict[Hashable, _ColorCodeAndName] = {}
 
     def get_code(
-        self, value: Tuple[int, int, int], exclude: Sequence[str] = ()
+        self, value: tuple[int, int, int], exclude: Sequence[str] = ()
     ) -> _ColorCodeAndName:
         """
         Return a (ansi_code, ansi_name) tuple. (E.g. ``(44, 'ansiblue')``.) for
@@ -180,9 +179,8 @@ class _16ColorCache:
         return cache[key]
 
     def _get(
-        self, value: Tuple[int, int, int], exclude: Sequence[str] = ()
+        self, value: tuple[int, int, int], exclude: Sequence[str] = ()
     ) -> _ColorCodeAndName:
-
         r, g, b = value
         match = _get_closest_ansi_color(r, g, b, exclude=exclude)
 
@@ -202,7 +200,7 @@ class _256ColorCache(Dict[Tuple[int, int, int], int]):
 
     def __init__(self) -> None:
         # Build color table.
-        colors: List[Tuple[int, int, int]] = []
+        colors: list[tuple[int, int, int]] = []
 
         # colors 0..15: 16 basic colors
         colors.append((0x00, 0x00, 0x00))  # 0
@@ -238,7 +236,7 @@ class _256ColorCache(Dict[Tuple[int, int, int], int]):
 
         self.colors = colors
 
-    def __missing__(self, value: Tuple[int, int, int]) -> int:
+    def __missing__(self, value: tuple[int, int, int]) -> int:
         r, g, b = value
 
         # Find closest color.
@@ -269,7 +267,8 @@ _256_colors = _256ColorCache()
 class _EscapeCodeCache(Dict[Attrs, str]):
     """
     Cache for VT100 escape codes. It maps
-    (fgcolor, bgcolor, bold, underline, reverse) tuples to VT100 escape sequences.
+    (fgcolor, bgcolor, bold, underline, strike, reverse) tuples to VT100
+    escape sequences.
 
     :param true_color: When True, use 24bit colors instead of 256 colors.
     """
@@ -278,8 +277,18 @@ class _EscapeCodeCache(Dict[Attrs, str]):
         self.color_depth = color_depth
 
     def __missing__(self, attrs: Attrs) -> str:
-        fgcolor, bgcolor, bold, underline, italic, blink, reverse, hidden = attrs
-        parts: List[str] = []
+        (
+            fgcolor,
+            bgcolor,
+            bold,
+            underline,
+            strike,
+            italic,
+            blink,
+            reverse,
+            hidden,
+        ) = attrs
+        parts: list[str] = []
 
         parts.extend(self._colors_to_code(fgcolor or "", bgcolor or ""))
 
@@ -295,6 +304,8 @@ class _EscapeCodeCache(Dict[Attrs, str]):
             parts.append("7")
         if hidden:
             parts.append("8")
+        if strike:
+            parts.append("9")
 
         if parts:
             result = "\x1b[0;" + ";".join(parts) + "m"
@@ -304,7 +315,7 @@ class _EscapeCodeCache(Dict[Attrs, str]):
         self[attrs] = result
         return result
 
-    def _color_name_to_rgb(self, color: str) -> Tuple[int, int, int]:
+    def _color_name_to_rgb(self, color: str) -> tuple[int, int, int]:
         "Turn 'ffffff', into (0xff, 0xff, 0xff)."
         try:
             rgb = int(color, 16)
@@ -325,7 +336,7 @@ class _EscapeCodeCache(Dict[Attrs, str]):
         # same. (Unless they were explicitly defined to be the same color.)
         fg_ansi = ""
 
-        def get(color: str, bg: bool) -> List[int]:
+        def get(color: str, bg: bool) -> list[int]:
             nonlocal fg_ansi
 
             table = BG_ANSI_COLORS if bg else FG_ANSI_COLORS
@@ -367,14 +378,14 @@ class _EscapeCodeCache(Dict[Attrs, str]):
                 else:
                     return [(48 if bg else 38), 5, _256_colors[rgb]]
 
-        result: List[int] = []
+        result: list[int] = []
         result.extend(get(fg_color, False))
         result.extend(get(bg_color, True))
 
         return map(str, result)
 
 
-def _get_size(fileno: int) -> Tuple[int, int]:
+def _get_size(fileno: int) -> tuple[int, int]:
     """
     Get the size of this pseudo terminal.
 
@@ -390,59 +401,65 @@ class Vt100_Output(Output):
     :param get_size: A callable which returns the `Size` of the output terminal.
     :param stdout: Any object with has a `write` and `flush` method + an 'encoding' property.
     :param term: The terminal environment variable. (xterm, xterm-256color, linux, ...)
-    :param write_binary: Encode the output before writing it. If `True` (the
-        default), the `stdout` object is supposed to expose an `encoding` attribute.
+    :param enable_cpr: When `True` (the default), send "cursor position
+        request" escape sequences to the output in order to detect the cursor
+        position. That way, we can properly determine how much space there is
+        available for the UI (especially for drop down menus) to render. The
+        `Renderer` will still try to figure out whether the current terminal
+        does respond to CPR escapes. When `False`, never attempt to send CPR
+        requests.
     """
 
     # For the error messages. Only display "Output is not a terminal" once per
     # file descriptor.
-    _fds_not_a_terminal: Set[int] = set()
+    _fds_not_a_terminal: set[int] = set()
 
     def __init__(
         self,
         stdout: TextIO,
         get_size: Callable[[], Size],
-        term: Optional[str] = None,
-        write_binary: bool = True,
-        default_color_depth: Optional[ColorDepth] = None,
+        term: str | None = None,
+        default_color_depth: ColorDepth | None = None,
         enable_bell: bool = True,
+        enable_cpr: bool = True,
     ) -> None:
-
         assert all(hasattr(stdout, a) for a in ("write", "flush"))
 
-        if write_binary:
-            assert hasattr(stdout, "encoding")
-
-        self._buffer: List[str] = []
+        self._buffer: list[str] = []
         self.stdout: TextIO = stdout
-        self.write_binary = write_binary
         self.default_color_depth = default_color_depth
         self._get_size = get_size
         self.term = term
         self.enable_bell = enable_bell
+        self.enable_cpr = enable_cpr
 
         # Cache for escape codes.
-        self._escape_code_caches: Dict[ColorDepth, _EscapeCodeCache] = {
+        self._escape_code_caches: dict[ColorDepth, _EscapeCodeCache] = {
             ColorDepth.DEPTH_1_BIT: _EscapeCodeCache(ColorDepth.DEPTH_1_BIT),
             ColorDepth.DEPTH_4_BIT: _EscapeCodeCache(ColorDepth.DEPTH_4_BIT),
             ColorDepth.DEPTH_8_BIT: _EscapeCodeCache(ColorDepth.DEPTH_8_BIT),
             ColorDepth.DEPTH_24_BIT: _EscapeCodeCache(ColorDepth.DEPTH_24_BIT),
         }
 
+        # Keep track of whether the cursor shape was ever changed.
+        # (We don't restore the cursor shape if it was never changed - by
+        # default, we don't change them.)
+        self._cursor_shape_changed = False
+
     @classmethod
     def from_pty(
         cls,
         stdout: TextIO,
-        term: Optional[str] = None,
-        default_color_depth: Optional[ColorDepth] = None,
+        term: str | None = None,
+        default_color_depth: ColorDepth | None = None,
         enable_bell: bool = True,
-    ) -> "Vt100_Output":
+    ) -> Vt100_Output:
         """
         Create an Output class from a pseudo terminal.
         (This will take the dimensions by reading the pseudo
         terminal attributes.)
         """
-        fd: Optional[int]
+        fd: int | None
         # Normally, this requires a real TTY device, but people instantiate
         # this class often during unit tests as well. For convenience, we print
         # an error message, use standard dimensions, and go on.
@@ -537,6 +554,9 @@ class Vt100_Output(Output):
     def enable_mouse_support(self) -> None:
         self.write_raw("\x1b[?1000h")
 
+        # Enable mouse-drag support.
+        self.write_raw("\x1b[?1003h")
+
         # Enable urxvt Mouse mode. (For terminals that understand this.)
         self.write_raw("\x1b[?1015h")
 
@@ -550,6 +570,7 @@ class Vt100_Output(Output):
         self.write_raw("\x1b[?1000l")
         self.write_raw("\x1b[?1015l")
         self.write_raw("\x1b[?1006l")
+        self.write_raw("\x1b[?1003l")
 
     def erase_end_of_line(self) -> None:
         """
@@ -645,6 +666,31 @@ class Vt100_Output(Output):
     def show_cursor(self) -> None:
         self.write_raw("\x1b[?12l\x1b[?25h")  # Stop blinking cursor and show.
 
+    def set_cursor_shape(self, cursor_shape: CursorShape) -> None:
+        if cursor_shape == CursorShape._NEVER_CHANGE:
+            return
+
+        self._cursor_shape_changed = True
+        self.write_raw(
+            {
+                CursorShape.BLOCK: "\x1b[2 q",
+                CursorShape.BEAM: "\x1b[6 q",
+                CursorShape.UNDERLINE: "\x1b[4 q",
+                CursorShape.BLINKING_BLOCK: "\x1b[1 q",
+                CursorShape.BLINKING_BEAM: "\x1b[5 q",
+                CursorShape.BLINKING_UNDERLINE: "\x1b[3 q",
+            }.get(cursor_shape, "")
+        )
+
+    def reset_cursor_shape(self) -> None:
+        "Reset cursor shape."
+        # (Only reset cursor shape, if we ever changed it.)
+        if self._cursor_shape_changed:
+            self._cursor_shape_changed = False
+
+            # Reset cursor shape.
+            self.write_raw("\x1b[0 q")
+
     def flush(self) -> None:
         """
         Write to output stream and flush.
@@ -655,44 +701,7 @@ class Vt100_Output(Output):
         data = "".join(self._buffer)
         self._buffer = []
 
-        try:
-            # Ensure that `self.stdout` is made blocking when writing into it.
-            # Otherwise, when uvloop is activated (which makes stdout
-            # non-blocking), and we write big amounts of text, then we get a
-            # `BlockingIOError` here.
-            with blocking_io(self.stdout):
-                # (We try to encode ourself, because that way we can replace
-                # characters that don't exist in the character set, avoiding
-                # UnicodeEncodeError crashes. E.g. u'\xb7' does not appear in 'ascii'.)
-                # My Arch Linux installation of july 2015 reported 'ANSI_X3.4-1968'
-                # for sys.stdout.encoding in xterm.
-                out: IO
-                if self.write_binary:
-                    if hasattr(self.stdout, "buffer"):
-                        out = self.stdout.buffer  # Py3.
-                    else:
-                        out = self.stdout
-                    out.write(data.encode(self.stdout.encoding or "utf-8", "replace"))
-                else:
-                    self.stdout.write(data)
-
-                self.stdout.flush()
-        except IOError as e:
-            if e.args and e.args[0] == errno.EINTR:
-                # Interrupted system call. Can happen in case of a window
-                # resize signal. (Just ignore. The resize handler will render
-                # again anyway.)
-                pass
-            elif e.args and e.args[0] == 0:
-                # This can happen when there is a lot of output and the user
-                # sends a KeyboardInterrupt by pressing Control-C. E.g. in
-                # a Python REPL when we execute "while True: print('test')".
-                # (The `ptpython` REPL uses this `Output` class instead of
-                # `stdout` directly -- in order to be network transparent.)
-                # So, just ignore.
-                pass
-            else:
-                raise
+        flush_stdout(self.stdout, data)
 
     def ask_for_cpr(self) -> None:
         """
@@ -703,6 +712,9 @@ class Vt100_Output(Output):
 
     @property
     def responds_to_cpr(self) -> bool:
+        if not self.enable_cpr:
+            return False
+
         # When the input is a tty, we assume that CPR is supported.
         # It's not when the input is piped from Pexpect.
         if os.environ.get("PROMPT_TOOLKIT_NO_CPR", "") == "1":
@@ -744,37 +756,3 @@ class Vt100_Output(Output):
             return ColorDepth.DEPTH_4_BIT
 
         return ColorDepth.DEFAULT
-
-
-@contextmanager
-def blocking_io(io: IO[str]) -> Iterator[None]:
-    """
-    Ensure that the FD for `io` is set to blocking in here.
-    """
-    if sys.platform == "win32":
-        # On Windows, the `os` module doesn't have a `get/set_blocking`
-        # function.
-        yield
-        return
-
-    try:
-        fd = io.fileno()
-        blocking = os.get_blocking(fd)
-    except:  # noqa
-        # Failed somewhere.
-        # `get_blocking` can raise `OSError`.
-        # The io object can raise `AttributeError` when no `fileno()` method is
-        # present if we're not a real file object.
-        blocking = True  # Assume we're good, and don't do anything.
-
-    try:
-        # Make blocking if we weren't blocking yet.
-        if not blocking:
-            os.set_blocking(fd, True)
-
-        yield
-
-    finally:
-        # Restore original blocking mode.
-        if not blocking:
-            os.set_blocking(fd, blocking)
