@@ -2,6 +2,8 @@
 Formatter classes for the progress bar.
 Each progress bar consists of a list of these formatters.
 """
+from __future__ import annotations
+
 import datetime
 import time
 from abc import ABCMeta, abstractmethod
@@ -45,13 +47,13 @@ class Formatter(metaclass=ABCMeta):
     @abstractmethod
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
         pass
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         return D()
 
 
@@ -65,13 +67,13 @@ class Text(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
         return self.text
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         return fragment_list_width(self.text)
 
 
@@ -95,11 +97,10 @@ class Label(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
-
         label = self._add_suffix(progress.label)
         cwidth = fragment_list_width(label)
 
@@ -112,7 +113,7 @@ class Label(Formatter):
 
         return label
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         if self.width:
             return self.width
 
@@ -133,14 +134,13 @@ class Percentage(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
-
         return HTML(self.template).format(percentage=round(progress.percentage, 1))
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         return D.exact(6)
 
 
@@ -160,7 +160,6 @@ class Bar(Formatter):
         sym_c: str = " ",
         unknown: str = "#",
     ) -> None:
-
         assert len(sym_a) == 1 and get_cwidth(sym_a) == 1
         assert len(sym_c) == 1 and get_cwidth(sym_c) == 1
 
@@ -173,31 +172,41 @@ class Bar(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
+        if progress.done or progress.total or progress.stopped:
+            sym_a, sym_b, sym_c = self.sym_a, self.sym_b, self.sym_c
 
-        # Subtract left, bar_b and right.
-        width -= get_cwidth(self.start + self.sym_b + self.end)
-
-        if progress.total:
-            pb_a = int(progress.percentage * width / 100)
-            bar_a = self.sym_a * pb_a
-            bar_b = self.sym_b
-            bar_c = self.sym_c * (width - pb_a)
+            # Compute pb_a based on done, total, or stopped states.
+            if progress.done:
+                # 100% completed irrelevant of how much was actually marked as completed.
+                percent = 1.0
+            else:
+                # Show percentage completed.
+                percent = progress.percentage / 100
         else:
-            # Total is unknown.
-            pb_a = int(time.time() * 20 % 100 * width / 100)
-            bar_a = self.sym_c * pb_a
-            bar_b = self.unknown
-            bar_c = self.sym_c * (width - pb_a)
+            # Total is unknown and bar is still running.
+            sym_a, sym_b, sym_c = self.sym_c, self.unknown, self.sym_c
+
+            # Compute percent based on the time.
+            percent = time.time() * 20 % 100 / 100
+
+        # Subtract left, sym_b, and right.
+        width -= get_cwidth(self.start + sym_b + self.end)
+
+        # Scale percent by width
+        pb_a = int(percent * width)
+        bar_a = sym_a * pb_a
+        bar_b = sym_b
+        bar_c = sym_c * (width - pb_a)
 
         return HTML(self.template).format(
             start=self.start, end=self.end, bar_a=bar_a, bar_b=bar_b, bar_c=bar_c
         )
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         return D(min=9)
 
 
@@ -210,18 +219,17 @@ class Progress(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
-
         return HTML(self.template).format(
             current=progress.items_completed, total=progress.total or "?"
         )
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         all_lengths = [
-            len("{0:>3}".format(c.total or "?")) for c in progress_bar.counters
+            len("{:>3}".format(c.total or "?")) for c in progress_bar.counters
         ]
         all_lengths.append(1)
         return D.exact(max(all_lengths) * 2 + 1)
@@ -231,7 +239,7 @@ def _format_timedelta(timedelta: datetime.timedelta) -> str:
     """
     Return hh:mm:ss, or mm:ss if the amount of hours is zero.
     """
-    result = "{0}".format(timedelta).split(".")[0]
+    result = f"{timedelta}".split(".")[0]
     if result.startswith("0:"):
         result = result[2:]
     return result
@@ -244,17 +252,16 @@ class TimeElapsed(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
-
         text = _format_timedelta(progress.time_elapsed).rjust(width)
         return HTML("<time-elapsed>{time_elapsed}</time-elapsed>").format(
             time_elapsed=text
         )
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         all_values = [
             len(_format_timedelta(c.time_elapsed)) for c in progress_bar.counters
         ]
@@ -273,11 +280,10 @@ class TimeLeft(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
-
         time_left = progress.time_left
         if time_left is not None:
             formatted_time_left = _format_timedelta(time_left)
@@ -286,7 +292,7 @@ class TimeLeft(Formatter):
 
         return HTML(self.template).format(time_left=formatted_time_left.rjust(width))
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         all_values = [
             len(_format_timedelta(c.time_left)) if c.time_left is not None else 7
             for c in progress_bar.counters
@@ -307,17 +313,16 @@ class IterationsPerSecond(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
-
         value = progress.items_completed / progress.time_elapsed.total_seconds()
         return HTML(self.template.format(iterations_per_second=value))
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         all_values = [
-            len("{0:.2f}".format(c.items_completed / c.time_elapsed.total_seconds()))
+            len(f"{c.items_completed / c.time_elapsed.total_seconds():.2f}")
             for c in progress_bar.counters
         ]
         if all_values:
@@ -334,21 +339,20 @@ class SpinningWheel(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
-
         index = int(time.time() * 3) % len(self.characters)
         return HTML("<spinning-wheel>{0}</spinning-wheel>").format(
             self.characters[index]
         )
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         return D.exact(1)
 
 
-def _hue_to_rgb(hue: float) -> Tuple[int, int, int]:
+def _hue_to_rgb(hue: float) -> tuple[int, int, int]:
     """
     Take hue between 0 and 1, return (r, g, b).
     """
@@ -382,11 +386,10 @@ class Rainbow(Formatter):
 
     def format(
         self,
-        progress_bar: "ProgressBar",
-        progress: "ProgressBarCounter[object]",
+        progress_bar: ProgressBar,
+        progress: ProgressBarCounter[object],
         width: int,
     ) -> AnyFormattedText:
-
         # Get formatted text from nested formatter, and explode it in
         # text/style tuples.
         result = self.formatter.format(progress_bar, progress, width)
@@ -402,11 +405,11 @@ class Rainbow(Formatter):
             )
         return result2
 
-    def get_width(self, progress_bar: "ProgressBar") -> AnyDimension:
+    def get_width(self, progress_bar: ProgressBar) -> AnyDimension:
         return self.formatter.get_width(progress_bar)
 
 
-def create_default_formatters() -> List[Formatter]:
+def create_default_formatters() -> list[Formatter]:
     """
     Return the list of default formatters.
     """
