@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import os
 import signal
 import sys
@@ -65,10 +67,10 @@ class Event(Generic[_Sender]):
     """
 
     def __init__(
-        self, sender: _Sender, handler: Optional[Callable[[_Sender], None]] = None
+        self, sender: _Sender, handler: Callable[[_Sender], None] | None = None
     ) -> None:
         self.sender = sender
-        self._handlers: List[Callable[[_Sender], None]] = []
+        self._handlers: list[Callable[[_Sender], None]] = []
 
         if handler is not None:
             self += handler
@@ -98,14 +100,14 @@ class Event(Generic[_Sender]):
         if handler in self._handlers:
             self._handlers.remove(handler)
 
-    def __iadd__(self, handler: Callable[[_Sender], None]) -> "Event[_Sender]":
+    def __iadd__(self, handler: Callable[[_Sender], None]) -> Event[_Sender]:
         """
         `event += handler` notation for adding a handler.
         """
         self.add_handler(handler)
         return self
 
-    def __isub__(self, handler: Callable[[_Sender], None]) -> "Event[_Sender]":
+    def __isub__(self, handler: Callable[[_Sender], None]) -> Event[_Sender]:
         """
         `event -= handler` notation for removing a handler.
         """
@@ -188,24 +190,27 @@ def is_windows() -> bool:
     """
     True when we are using Windows.
     """
-    return sys.platform.startswith("win")  # E.g. 'win32', not 'darwin' or 'linux2'
+    return sys.platform == "win32"  # Not 'darwin' or 'linux2'
 
 
 def is_windows_vt100_supported() -> bool:
     """
     True when we are using Windows, but VT100 escape sequences are supported.
     """
-    # Import needs to be inline. Windows libraries are not always available.
-    from prompt_toolkit.output.windows10 import is_win_vt100_enabled
+    if sys.platform == "win32":
+        # Import needs to be inline. Windows libraries are not always available.
+        from prompt_toolkit.output.windows10 import is_win_vt100_enabled
 
-    return is_windows() and is_win_vt100_enabled()
+        return is_win_vt100_enabled()
+
+    return False
 
 
 def is_conemu_ansi() -> bool:
     """
     True when the ConEmu Windows console is used.
     """
-    return is_windows() and os.environ.get("ConEmuANSI", "OFF") == "ON"
+    return sys.platform == "win32" and os.environ.get("ConEmuANSI", "OFF") == "ON"
 
 
 def in_main_thread() -> bool:
@@ -232,7 +237,7 @@ _T = TypeVar("_T")
 
 
 def take_using_weights(
-    items: List[_T], weights: List[int]
+    items: list[_T], weights: list[int]
 ) -> Generator[_T, None, None]:
     """
     Generator that keeps yielding items from the items list, in proportion to
@@ -285,7 +290,7 @@ def take_using_weights(
         i += 1
 
 
-def to_str(value: Union[Callable[[], str], str]) -> str:
+def to_str(value: Callable[[], str] | str) -> str:
     "Turn callable or string into string."
     if callable(value):
         return to_str(value())
@@ -293,7 +298,7 @@ def to_str(value: Union[Callable[[], str], str]) -> str:
         return str(value)
 
 
-def to_int(value: Union[Callable[[], int], int]) -> int:
+def to_int(value: Callable[[], int] | int) -> int:
     "Turn callable or int into int."
     if callable(value):
         return to_int(value())
@@ -312,7 +317,7 @@ def to_float(value: AnyFloat) -> float:
         return float(value)
 
 
-def is_dumb_terminal(term: Optional[str] = None) -> bool:
+def is_dumb_terminal(term: str | None = None) -> bool:
     """
     True if this terminal type is considered "dumb".
 
